package com.adc.da.system.service.impl;


import com.adc.da.dto.reqDTO.CheckCodeDetailDTO;
import com.adc.da.system.dao.CheckObdDetailMapper;
import com.adc.da.system.entity.CheckObdDetail;
import com.adc.da.system.entity.CheckObdOrg;
import com.adc.da.system.service.ICheckDetailObdService;
import com.adc.da.system.service.ICheckObdDetailService;
import com.adc.da.system.service.ICheckObdOrgService;
import com.adc.da.util.UUIDUtils;
import com.adc.da.excel.poi.excel.ExcelImportUtil;
import com.adc.da.excel.poi.excel.entity.ImportParams;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2021-04-01
 */
@Service
public class CheckObdDetailServiceImpl extends ServiceImpl<CheckObdDetailMapper, CheckObdDetail> implements ICheckObdDetailService {

//    private static final Logger logger = LoggerFactory.getLogger(FuelDateEntityServiceImpl.class);

//    @Autowired
//    private CheckObdDetailMapper checkObdDetailMapper;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private ICheckDetailObdService iCheckDetailObdService;
    @Autowired
    private ICheckObdOrgService iCheckObdOrgService;

    private static final HashMap<String, String> dataBeanFieldToDatabaseColumnMap = new HashMap<>();
    static {
        dataBeanFieldToDatabaseColumnMap.put("obdDetailName","OBD_DETAIL_NAME");
        dataBeanFieldToDatabaseColumnMap.put("userId","t3.USNAME");
        dataBeanFieldToDatabaseColumnMap.put("createTime","CREATE_TIME");
        dataBeanFieldToDatabaseColumnMap.put("baseId","t2.ORG_NAME");
        dataBeanFieldToDatabaseColumnMap.put("remarks","REMARKS");
        dataBeanFieldToDatabaseColumnMap.put("updateTime","UPDATE_TIME");
    }

    private String createTime = "CREATE_TIME";

    @Override
    public IPage queryDetail(Integer page, Integer limit, String obdDetailName, String userId, String baseId,
                             String sort, String isAsc ,String type) {
        Page iPage = new Page(page, limit);
        if ("normal".equals(isAsc)) {
            isAsc = "";
        }
        List<String> detailBaseList = new ArrayList<>();
        if (StringUtils.isNotBlank(baseId)){
            detailBaseList = Arrays.asList(baseId.split(","));
        }
        sort = dataBeanFieldToDatabaseColumnMap.get(sort);
        if (createTime.equalsIgnoreCase(sort) || StringUtils.isEmpty(sort)) {
            if (StringUtils.isEmpty(isAsc)) {
                isAsc = "DESC";
            }
            return baseMapper.queryDetail(iPage,obdDetailName,userId,detailBaseList,sort,isAsc,type);
        }
        return baseMapper.getQueryDetail(iPage,obdDetailName,userId,detailBaseList,sort,isAsc,type);
    }

    /**
     * 添加
     * @param checkObdDetail 核查维护项实体类
     * @return
     */
    @Override
    public Integer insertDetail(CheckObdDetail checkObdDetail) {
        String userId = request.getHeader("userId");
        checkObdDetail.setId(UUIDUtils.getUUID());
        checkObdDetail.setCreateTime(new Date());
        checkObdDetail.setUserId(userId);
        String base =checkObdDetail.getBaseId();
        List<String> detailBaseList = new ArrayList<>();
        if (StringUtils.isNotBlank(base)){
           detailBaseList = Arrays.asList(base.split(","));
        }
        List<CheckObdOrg> checkObdOrgList = new ArrayList<>();
        for (String baseId:detailBaseList) {
            CheckObdOrg checkObdOrg = new CheckObdOrg();
            checkObdOrg.setObdId(checkObdDetail.getId());
            checkObdOrg.setOrgId(baseId);
            checkObdOrgList.add(checkObdOrg);
        }
        iCheckObdOrgService.saveBatch(checkObdOrgList);
        checkObdDetail.setBaseId("");
        return baseMapper.insert(checkObdDetail);
    }

    /**
     * 修改
     * @param checkObdDetail 核查项维护实体类
     * @return 结果
     */
    @Override
    public Integer updateDetail(CheckObdDetail checkObdDetail) {
        checkObdDetail.setUpdateTime(new Date());
        QueryWrapper<CheckObdOrg> wrapper = new QueryWrapper<>();
        wrapper.eq("OBD_ID",checkObdDetail.getId());
        iCheckObdOrgService.remove(wrapper);
        String base =checkObdDetail.getBaseId();
        List<String> detailBaseList = new ArrayList<>();
        if (StringUtils.isNotBlank(base)){
            detailBaseList = Arrays.asList(base.split(","));
        }
        List<CheckObdOrg> checkObdOrgList = new ArrayList<>();
        for (String baseId:detailBaseList) {
            CheckObdOrg checkObdOrg = new CheckObdOrg();
            checkObdOrg.setObdId(checkObdDetail.getId());
            checkObdOrg.setOrgId(baseId);
            checkObdOrgList.add(checkObdOrg);
        }
        iCheckObdOrgService.saveBatch(checkObdOrgList);
        checkObdDetail.setBaseId("");
        return baseMapper.updateById(checkObdDetail);
    }

    /**
     * 删除
     * @param id 检索项目id
     * @return
     */
    @Override
    public Integer deleteDetail(String id) {
        List<String> idList = Arrays.asList(id.split(","));
        baseMapper.deleteDetailObd(idList);
        QueryWrapper<CheckObdOrg> wrapper = new QueryWrapper<>();
        wrapper.in("OBD_ID",idList);
        iCheckObdOrgService.remove(wrapper);
        return baseMapper.deleteBatchIds(idList);
    }

    @Override
    public Map<String, Object> importObdDetail(MultipartFile file, String userId, String baseId, String type) {
        List<String> baseList = Arrays.asList(baseId.split(","));

        boolean flag = true;

        ImportParams params = new ImportParams();

        //需要插入数据库的集合
        List<CheckObdDetail> successList = new ArrayList<>();
        //表格内重复检查项目名称
        List<String> errorList = new ArrayList<>();
        //检查项目名称长度超出限制的
        List<String> errorList2 = new ArrayList<>();
        List<String> excelErrorList = new ArrayList<>();
        //无论是否成功插入数据库，都返回map
        Map<String, Object> map = new HashMap<>();
        List<String> excelData = new ArrayList<>();
        String fileName = file.getOriginalFilename();
        InputStream is = null;
        try{
            is = file.getInputStream();
            if(is == null){
                return null;
            }
            Workbook workbook = null;
            if (fileName.endsWith("xlsx")) {
                workbook = new XSSFWorkbook(is);
            } else {
                workbook = new HSSFWorkbook(is);
            }
            //获取Excel文件第一个sheet
            Sheet sheet = workbook.getSheetAt(0);
            String title[];
            if ("2".equals(type)){
                title = new String[]{"检查项目","总成名称","备注"};
            } else {
                title = new String[]{"检查项目","备注"};
            }
            if(!(sheet.getRow(0) !=null && title[0].equals(sheet.getRow(0).getCell(0).getStringCellValue())
                    && title[1].equals(sheet.getRow(0).getCell(1).getStringCellValue()))){
                return null;
            }
            List<CheckCodeDetailDTO> excelList  //获取excel中的内容
                    = ExcelImportUtil.importExcel(file.getInputStream(), CheckCodeDetailDTO.class, params);
            List<String> name = baseMapper.selectName(baseList,type);
            CheckObdDetail checkObdDetail;
            if(excelList.size() == 0){
                map.put("ObdDetailName",1);
                return map;
            }
            for (CheckCodeDetailDTO c :excelList) {
                if(StringUtils.isBlank(c.getObdDetailName())){
                    map.put("ObdDetailName",1);
                    return map;
                }
                if("2".equals(type) && StringUtils.isBlank(c.getProjectName())){
                    map.put("projectName",1);
                    return map;
                }
                if(c.getObdDetailName().length()>1000){
                    map.put("FieldLength",1);
                    return map;
                }
                if (excelData.contains(c.getObdDetailName())) {
                    excelErrorList.add(c.getObdDetailName());
                    continue;
                } else {
                    excelData.add(c.getObdDetailName());
                }
                if(name.contains(c.getObdDetailName())){
                    errorList.add(c.getObdDetailName());
                }else {
                    checkObdDetail = new CheckObdDetail();
                    checkObdDetail.setId(UUIDUtils.getUUID());
//                    checkObdDetail.setBaseId(baseId);
                    List<String> detailBaseList = new ArrayList<>();
                    if (StringUtils.isNotBlank(baseId)){
                        detailBaseList = Arrays.asList(baseId.split(","));
                    }
                    List<CheckObdOrg> checkObdOrgList = new ArrayList<>();
                    for (String base:detailBaseList) {
                        CheckObdOrg checkObdOrg = new CheckObdOrg();
                        checkObdOrg.setObdId(checkObdDetail.getId());
                        checkObdOrg.setOrgId(base);
                        checkObdOrgList.add(checkObdOrg);
                    }
                    iCheckObdOrgService.saveBatch(checkObdOrgList);
                    checkObdDetail.setCreateTime(new Date());
                    checkObdDetail.setUserId(userId);
                    checkObdDetail.setProjectName(c.getProjectName());
                    checkObdDetail.setObdDetailName(c.getObdDetailName());
                    checkObdDetail.setRemarks(c.getRemarks());
                    checkObdDetail.setType(type);
                    successList.add(checkObdDetail);
                }
            }
            if (!successList.isEmpty()) {
                this.saveBatch(successList);
            }
            map.put("successList", successList);
            map.put("errorList", errorList);
            map.put("errorList2", errorList2);
            map.put("excelErrorList",excelErrorList);
            return map;
        } catch (Exception e) {
            //日志打印
//            logger.error("导入失败", e);
            return null;
//            throw new AdcDaBaseException("上传文件内容格式不对，请修改后重试");
        }
    }

    @Override
    public int selectCountByItemAndBase(String baseId, String detailType, String detailName, String id) {
//        QueryWrapper<CheckObdDetail> wrapper= new QueryWrapper<CheckObdDetail>();
//        wrapper.eq("BASE_ID",baseId);
//        wrapper.eq("TYPE",detailType);
//        wrapper.eq("OBD_DETAIL_NAME",detailName);
//        if(StringUtils.isNotBlank(id)){
//            wrapper.notIn("ID",id);
//        }
        List<String> detailBaseList = new ArrayList<>();
        if (StringUtils.isNotBlank(baseId)){
            detailBaseList = Arrays.asList(baseId.split(","));
        }
        return baseMapper.selectCountByItemAndBase(detailBaseList,detailType,detailName,id);
    }

    @Override
    public List<CheckObdDetail> getDetailsByMouldIdAndType(String obdId, String detailType) {
        List<CheckObdDetail> checkObdDetailList = baseMapper.selectDetailByObdId(obdId);
        return checkObdDetailList!= null && !checkObdDetailList.isEmpty()?checkObdDetailList:new ArrayList<>();
    }

    @Override
    public List<CheckObdDetail> getDetailListByType(String baseId, String detailType, String obdId, String remarks) {
//        QueryWrapper wrapper = new QueryWrapper();
//        wrapper.eq("TYPE",detailType);
//        wrapper.eq("BASE_ID",baseId);
//        wrapper.orderByDesc("CREATE_TIME");
        List<String> detailBaseList = new ArrayList<>();
        if (StringUtils.isNotBlank(baseId)){
            detailBaseList = Arrays.asList(baseId.split(","));
        }
        List<CheckObdDetail> list = baseMapper.getDetailListByType(detailBaseList,detailType,remarks);
        List<String> obdList = new ArrayList<>();
        if (StringUtils.isNotBlank(obdId)) {
            obdList = iCheckDetailObdService.getDetailIdByMould(obdId);
        }
        for (CheckObdDetail checkObdDetail:list) {
            if (obdList.contains(checkObdDetail.getId())) {
                checkObdDetail.setIsSelect("0");
            } else {
                checkObdDetail.setIsSelect("1");
            }
        }
        return list;
    }

    @Override
    public List<String> queryZcName(String scanCodeId) {
//        QueryWrapper<CheckObdDetail> wrapper = new QueryWrapper();
//        wrapper.select("distinct PROJECT_NAME");
//        wrapper.isNotNull("FAC_CODE");
//        wrapper.isNotNull("MODEL_CODE");
//        wrapper.isNotNull("PROJECT_NAME");
        return baseMapper.queryZcName(scanCodeId);
    }

    @Override
    public List<CheckObdDetail> listByProductName(String assemblyName, String modelSeriesListId) {
//        QueryWrapper<CheckObdDetail> wrapper = new QueryWrapper();
//        wrapper.eq("PROJECT_NAME",assemblyName);
//        wrapper.isNotNull("FAC_CODE");
//        wrapper.isNotNull("MODEL_CODE");
        return baseMapper.listByProductName(assemblyName,modelSeriesListId);
    }

}
