package com.adc.da.system.service.impl;


import com.adc.da.system.dao.CheckObdMapper;
import com.adc.da.system.entity.CheckDetailObd;
import com.adc.da.system.entity.CheckObd;
import com.adc.da.system.entity.CheckObdOrg;
import com.adc.da.system.entity.TsOrg;
import com.adc.da.system.service.ICheckDetailObdService;
import com.adc.da.system.service.ICheckObdOrgService;
import com.adc.da.system.service.ICheckObdService;
import com.adc.da.util.UUIDUtils;
import com.adc.da.util.utils.UserUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * OBD参数审核表 服务实现类
 * </p>
 *
 * @author jobob
 * @since 2021-03-30
 */
@Service
public class CheckObdServiceImpl extends ServiceImpl<CheckObdMapper, CheckObd> implements ICheckObdService {

    @Autowired
    private CheckObdMapper checkObdMapper;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private ICheckObdOrgService obdOrgService;
    @Autowired
    private ICheckDetailObdService detailObdService;
    @Autowired
    private ICheckObdOrgService iCheckObdOrgService;

    private static final HashMap<String, String> dataBeanFieldToDatabaseColumnMap = new HashMap<>();
    static {
        dataBeanFieldToDatabaseColumnMap.put("obdName","OBD_NAME");
        dataBeanFieldToDatabaseColumnMap.put("operator","t3.USNAME");
        dataBeanFieldToDatabaseColumnMap.put("creationTime","CREATION_TIME");
        dataBeanFieldToDatabaseColumnMap.put("base","t2.ORG_NAME");
        dataBeanFieldToDatabaseColumnMap.put("remarks","REMARKS");
        dataBeanFieldToDatabaseColumnMap.put("changeTime","CHANGE_TIME");
    }

    private String creationTime = "CREATION_TIME";

    /**
     * 模糊查询（带有分页和排序）
     * @param page 页
     * @param limit 长度
     * @param obdName 模板名称
     * @param operator 创建人
     * @param base 事业部
     * @param sort 排序字段
     * @param isAsc 是否正序
     * @param userBase
     * @return 查询结果
     */
    @Override
    public IPage queryCheckObd(Integer page, Integer limit, String obdName, String operator, String base, String sort,
                               String isAsc, String type, String userBase,String startTime,String endTime) {
        Page iPage = new Page(page, limit);
        if ("normal".equals(isAsc)) {
            isAsc = "";
        }
        List<String> list = new ArrayList<>();
        if(StringUtils.isNotBlank(base)){
                list = checkObdMapper.selectObd(base);
            if(list.size() == 0){
                IPage<CheckObd> checkObdIPage = iPage;
                checkObdIPage.setTotal(0);
                checkObdIPage.setRecords(new ArrayList<>());
                return checkObdIPage;
            }
        }
//        List<String> userBaseForList = new ArrayList<>();
//        if(StringUtils.isNotBlank(userBase)){
//            userBaseForList = checkObdMapper.selectObd(userBase);
//            if(userBaseForList.size() == 0){
//                IPage<CheckObd> checkObdIPage = iPage;
//                checkObdIPage.setTotal(0);
//                checkObdIPage.setRecords(new ArrayList<>());
//                return checkObdIPage;
//            }
//        }
//        if (StringUtils.isNotBlank(userBase)) {
//            list = userBaseForList;
//        }
        if(StringUtils.isNotBlank(sort)){
            switch (sort){
                case "obdName":
                    sort = "NLSSORT(OBD_NAME,'NLS_SORT = SCHINESE_PINYIN_M')";
                    break;
                case "operator":
                    sort = "NLSSORT(OPERATOR,'NLS_SORT = SCHINESE_PINYIN_M')";
                    break;
                case "creationTime":
                    sort = "t1.CREATION_TIME";
                    break;
                case "changeTime":
                    sort = "t1.CHANGE_TIME";
                    break;
                case "distributionName":
                    sort = "t1.ID";
                    break;
                case "remarks":
                    sort = "NLSSORT(REMARKS,'NLS_SORT = SCHINESE_PINYIN_M')";
                    break;
                default:
                    sort = "t1.CREATION_TIME";
                    isAsc = "asc";
            }
        }else {
            sort = "t1.CREATION_TIME";
            isAsc = "DESC";
        }
        if (isAsc != null && "normal".equals(isAsc)) {
            isAsc = null;
        }
            return checkObdMapper.queryCheckObd(iPage,obdName,operator,sort,isAsc,type,startTime,endTime,list);
    }

    @Override
    public String selectDetailObd(String id) {
        return checkObdMapper.selectDetailObd(id);
    }

    @Override
    public List<TsOrg> selectOrg() {
        return checkObdMapper.selectOrg();
    }

    /**
     * 添加
     * @return 添加结果
     */
    @Override
    public Integer insertObd(CheckObd checkObd) {
        checkObd.setMouldType("0");
        String userId = request.getHeader("userId");
        checkObd.setId(UUIDUtils.getUUID());
        checkObd.setOperator(userId);
        checkObd.setCreationTime(new Date());
        if(StringUtils.isNotBlank(checkObd.getDetailId())){
            List<String> detailIdList = Arrays.asList(checkObd.getDetailId().split(","));
            List<CheckDetailObd> list = new ArrayList<>();
            CheckDetailObd checkDetailObd;
            for (String s : detailIdList) {
                checkDetailObd = new CheckDetailObd();
                checkDetailObd.setObdId(checkObd.getId());
                checkDetailObd.setDetailId(s);
                list.add(checkDetailObd);
            }
            detailObdService.insertDetailObd(list);
        }
        if(StringUtils.isNotBlank(checkObd.getDistributionBase())) {
            List<String> distributionBaseList = Arrays.asList(checkObd.getDistributionBase().split(","));
            List<CheckObdOrg> list = new ArrayList<>();
            CheckObdOrg checkObdOrg;
            for (String s : distributionBaseList) {
                checkObdOrg = new CheckObdOrg();
                checkObdOrg.setObdId(checkObd.getId());
                checkObdOrg.setOrgId(s);
                list.add(checkObdOrg);
            }
            obdOrgService.insertObdOrg(list);
        }
        checkObd.setBase("");
        return baseMapper.insert(checkObd);
    }

    /**
     * 修改
     * @param checkObd obd实体类
     * @return
     */
    @Override
    public Integer updateObd(CheckObd checkObd) {
        checkObd.setChangeTime(new Date());
        checkObdMapper.deleteObdOrg(checkObd.getId());
        if(StringUtils.isNotBlank(checkObd.getDistributionBase())) {
            List<String> distributionBaseList = Arrays.asList(checkObd.getDistributionBase().split(","));
            distributionBaseList = distributionBaseList.stream().distinct().collect(Collectors.toList());
            List<CheckObdOrg> list = new ArrayList<>();
            CheckObdOrg checkObdOrg;
            for (String s : distributionBaseList) {
                checkObdOrg = new CheckObdOrg();
                checkObdOrg.setObdId(checkObd.getId());
                checkObdOrg.setOrgId(s);
                list.add(checkObdOrg);
            }
            obdOrgService.insertObdOrg(list);
        }
//        detailObdService.delByObdId(checkObd.getId());
        if(StringUtils.isNotBlank(checkObd.getDetailId())){
            List<String> detailIdList = Arrays.asList(checkObd.getDetailId().split(","));
            List<CheckDetailObd> list = new ArrayList<>();
            CheckDetailObd checkDetailObd;
            for (String s : detailIdList) {
                checkDetailObd = new CheckDetailObd();
                checkDetailObd.setObdId(checkObd.getId());
                checkDetailObd.setDetailId(s);
                list.add(checkDetailObd);
            }
//            detailObdService.insertDetailObd(list);
        }
        return checkObdMapper.updateObd(checkObd);
    }

    /**
     * 删除
     * @param id 模板id
     * @return
     */
    @Override
    public Integer deleteObd(String id) {
//        checkObdMapper.deleteDetailObd(id);
        List<String> idList = Arrays.asList(id.split(","));
        for (String s : idList) {
                checkObdMapper.deleteObdOrg(s);
        }
        return baseMapper.deleteBatchIds(idList);
    }

    @Override
    public List<CheckObd>  selectMouldByBase(String baseId, String mouldType, String standType) {
//        QueryWrapper wrapper = new QueryWrapper();
//        wrapper.eq("BASE",baseId);
//        wrapper.eq("TYPE",mouldType);
//        if (StringUtils.isNotBlank(standType)) {
//            wrapper.eq("MOULD_TYPE",standType);
//        }
        QueryWrapper<CheckObdOrg> mouldIdsWrapper = new QueryWrapper<>();
        mouldIdsWrapper.eq("ORG_ID",baseId);
        mouldIdsWrapper.select("OBD_ID");
        List mouldIds = iCheckObdOrgService.listObjs(mouldIdsWrapper);

        QueryWrapper<CheckObd> obdWrapper = new QueryWrapper<>();
        obdWrapper.in("ID",mouldIds);
        obdWrapper.eq("TYPE",mouldType);
        if (StringUtils.isNotBlank(standType)) {
            obdWrapper.eq("MOULD_TYPE",standType);
        }
        List<CheckObd> result= this.list(obdWrapper);
        return result != null && !result.isEmpty() ? result : new ArrayList<>();
    }
    @Override
    public int selectCountByBaseAndName(String baseId, String mouldType, String mouldName, String id){
        List<String> baseIds = Arrays.asList(baseId.split(","));
        return baseMapper.selectCountByBaseAndName(baseIds,mouldName,id,mouldType);
    }
}
