package com.adc.da.system.service.impl;


import com.adc.da.dto.resDTO.CerValidityDto;
import com.adc.da.dto.resDTO.VehicleDatabaseDetailsDto;
import com.adc.da.system.dao.VehicleDatabaseDao;
import com.adc.da.system.dao.VehicleDatabaseDetailsDao;
import com.adc.da.system.entity.VehicleDatabaseDetails;
import com.adc.da.system.service.IVehicleDatabaseDetailsService;
import com.adc.da.util.UUIDUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class VehicleDatabaseDetailsServiceImpl extends ServiceImpl<VehicleDatabaseDetailsDao, VehicleDatabaseDetails>
        implements IVehicleDatabaseDetailsService {

    @Autowired
    private VehicleDatabaseDao vehicleDatabaseDao;

    @Autowired
    private VehicleDatabaseDetailsDao vehicleDatabaseDetailsDao;

    @Override
    public IPage<VehicleDatabaseDetails> queryDetailsByPage(Page page, VehicleDatabaseDetails vehicleDatabaseDetails) {
        String orderBy = vehicleDatabaseDetails.getOrderBy();
        String order = vehicleDatabaseDetails.getOrder();
        if (orderBy != null) {
            switch (orderBy) {
                case "title":
                    orderBy = "TITLE";
                    break;
                case "vt":
                    orderBy = "NLSSORT(VT,'NLS_SORT = SCHINESE_PINYIN_M')";
                    break;
                case "detailData":
                    orderBy = "DETAILDATA";
                    break;
                case "ent":
                    orderBy = "NLSSORT(ENT,'NLS_SORT = SCHINESE_PINYIN_M')";
                    break;
                case "modelSpecification":
                    orderBy = "NLSSORT(modelSpecification1,'NLS_SORT = SCHINESE_PINYIN_M')";
                    break;
                case "factory":
                    orderBy = "NLSSORT(FACTORY,'NLS_SORT = SCHINESE_PINYIN_M')";
                    break;
                case "erpireDate":
                    orderBy = "ERPIRE_DATE";
                    break;
                case "status":
                    orderBy = "STATUS";
                    break;
                case "type":
                    orderBy = "TYPE";
                    break;
                default:
                    break;
            }
        } else {
            orderBy = "TITLE";
            order = "asc";
        }
        vehicleDatabaseDetails.setOrderBy(orderBy);
        vehicleDatabaseDetails.setOrder(order);
        List<String> baseIdList = new ArrayList<>();
        if (vehicleDatabaseDetails.getBase() != null) {
            baseIdList = Arrays.asList(vehicleDatabaseDetails.getBase().split(","));
        }
        List<VehicleDatabaseDetails> list = vehicleDatabaseDao.queryDetailsByPage(page, vehicleDatabaseDetails, baseIdList);
        for(VehicleDatabaseDetails vehicleDatabaseDetails1 : list) {
            vehicleDatabaseDetails1.setModelSpecification(vehicleDatabaseDetails1.getModelSpecificationAll());
        }
        page.setRecords(list);
        return page;
    }

    @Override
    public List<CerValidityDto> queryDetailsByList(String cccId, List<String> idList, String order, String orderBy, String modelCode, String base) {
        if (orderBy != null) {
            switch (orderBy) {
                case "title":
                    orderBy = "TITLE";
                    break;
                case "vt":
                    orderBy = "NLSSORT(VT,'NLS_SORT = SCHINESE_PINYIN_M')";
                    break;
                case "detailData":
                    orderBy = "DETAILDATA";
                    break;
                case "ent":
                    orderBy = "NLSSORT(ENT,'NLS_SORT = SCHINESE_PINYIN_M')";
                    break;
                case "modelSpecification":
                    orderBy = "NLSSORT(modelSpecification1,'NLS_SORT = SCHINESE_PINYIN_M')";
                    break;
                case "factory":
                    orderBy = "NLSSORT(FACTORY,'NLS_SORT = SCHINESE_PINYIN_M')";
                    break;
                case "erpireDate":
                    orderBy = "ERPIRE_DATE";
                    break;
                case "status":
                    orderBy = "STATUS";
                    break;
                case "type":
                    orderBy = "TYPE";
                    break;
                default:
                    break;
            }
        } else {
            orderBy = "TITLE";
            order = "asc";
        }
        List<String> baseIdList = new ArrayList<>();
        if (base != null) {
            baseIdList = Arrays.asList(base.split(","));
        }
        List<VehicleDatabaseDetails> list = vehicleDatabaseDao.queryDetailsByList(cccId, idList, order, orderBy, modelCode, baseIdList);
        List<CerValidityDto> cerValidityDtos = new ArrayList<>();
        for (VehicleDatabaseDetails data : list) {
            CerValidityDto cerValidityDto = new CerValidityDto();
            cerValidityDto.setTitle(data.getTitle());
            cerValidityDto.setVt(data.getVt());
            cerValidityDto.setDetailData(data.getDetailData());
            cerValidityDto.setEnt(data.getEnt());
            cerValidityDto.setModelSpecification(data.getModelSpecification());
            cerValidityDto.setFactory(data.getFactory());
            cerValidityDto.setErpireDate(data.getErpireDate());
            cerValidityDto.setStatus(data.getStatus());
            if (data.getType() == null) {
                cerValidityDto.setTypeRes("未核查");
            } else if ("0".equals(data.getType())) {
                cerValidityDto.setTypeRes("正常");
            } else if ("2".equals(data.getType())) {
                cerValidityDto.setTypeRes("异常");
            } else {
                cerValidityDto.setTypeRes("未核查");
            }
            cerValidityDtos.add(cerValidityDto);
        }
        return cerValidityDtos;
    }

    @Override
    public Map<String, Object> importDetails(MultipartFile file, String cccId, String modelCode, String base) {
        List<VehicleDatabaseDetailsDto> excelList = new ArrayList<>();
        //文件流
        String fileName = file.getOriginalFilename();
        InputStream is = null;
        try {
            is = file.getInputStream();
            Workbook workbook = null;
            if (fileName.endsWith("xlsx")) {
                workbook = new XSSFWorkbook(is);
            } else {
                workbook = new HSSFWorkbook(is);
            }
            //获取Excel文件第一个sheet
            Sheet sheet = workbook.getSheetAt(0);

            Map<String, Object> map =  this.importDetails2(sheet,excelList);
            if (map != null ) {
                if (!map.isEmpty())
                return map;
            }
        } catch (IOException e) {
            return null;
        }

        List<VehicleDatabaseDetails> successList = new ArrayList<>();
        List<String> errorList = new ArrayList<>();
        List<String> isNullList = new ArrayList<>();
        List<String> partNames = new ArrayList<>();
        List<String> cccNums = new ArrayList<>();
        try {
            this.importDetails4(excelList,partNames,cccNums,errorList,cccId,modelCode,base,successList,isNullList);
            if (this.saveBatch(successList)) {
                Map<String, Object> map = new HashMap<>();
                map.put("successList", successList);
                map.put("errorList", errorList);
                map.put("isNullList", isNullList);
                return map;
            } else {
                return null;
            }
        } catch (Exception e) {
            return null;
        }
    }

    private void importDetails4(List<VehicleDatabaseDetailsDto> excelList, List<String> partNames, List<String> cccNums, List<String> errorList, String cccId, String modelCode, String base, List<VehicleDatabaseDetails> successList, List<String> isNullList) {
        for (int i = 0; i < excelList.size(); i++) {
            this.importDetails5(excelList,i);
            if ((StringUtils.isNotEmpty(excelList.get(i).getPartName()) && !excelList.get(i).getPartName().equals("null"))
                    && (StringUtils.isNotEmpty(excelList.get(i).getCccNumCer()) && !excelList.get(i).getCccNumCer().equals("null"))) {
                if (partNames.contains(excelList.get(i).getPartName())
                        || cccNums.contains(excelList.get(i).getCccNumCer())) {
                    errorList.add(String.valueOf(i + 1));
                    continue;
                } else {
                    partNames.add(excelList.get(i).getPartName());
                    cccNums.add(excelList.get(i).getCccNumCer());
                }
                QueryWrapper queryWrapper = new QueryWrapper();
                queryWrapper.eq("PART_NAME", excelList.get(i).getPartName());
                queryWrapper.eq("CCC_ID", cccId);
                queryWrapper.eq("MODEL_CODE", modelCode);
                queryWrapper.eq("BASE", base);
                List<VehicleDatabaseDetails> details = vehicleDatabaseDetailsDao.selectList(queryWrapper);
                QueryWrapper queryWrapper1 = new QueryWrapper();
                queryWrapper1.eq("CCC_NUM_CER", excelList.get(i).getCccNumCer());
                queryWrapper1.eq("CCC_ID", cccId);
                queryWrapper1.eq("MODEL_CODE", modelCode);
                queryWrapper1.eq("BASE", base);
                List<VehicleDatabaseDetails> details1 = vehicleDatabaseDetailsDao.selectList(queryWrapper1);
                if (!details.isEmpty() || !details1.isEmpty()) {
                    errorList.add(String.valueOf(i + 1));
                } else {
                    VehicleDatabaseDetails vehicleDatabaseDetails = new VehicleDatabaseDetails();
                    vehicleDatabaseDetails.setId(UUIDUtils.getUUID());
                    vehicleDatabaseDetails.setPartName(excelList.get(i).getPartName());
                    vehicleDatabaseDetails.setCccId(cccId);
                    vehicleDatabaseDetails.setModelCode(modelCode);
                    vehicleDatabaseDetails.setPartNum(excelList.get(i).getPartNum());
                    vehicleDatabaseDetails.setCccNumCer(excelList.get(i).getCccNumCer());
                    vehicleDatabaseDetails.setManufacturer(excelList.get(i).getManufacturer());
                    vehicleDatabaseDetails.setBase(base);
                    if (successList.contains(vehicleDatabaseDetails)) {
                        errorList.add(String.valueOf(i + 1));
                    } else {
                        successList.add(vehicleDatabaseDetails);
                    }
                }
            } else {
                isNullList.add(String.valueOf(i + 1));
            }
        }

    }

    private void importDetails5(List<VehicleDatabaseDetailsDto> excelList, int i) {
        if (StringUtils.isEmpty(excelList.get(i).getCccNumCer()) || excelList.get(i).getCccNumCer().equals(
                "null")) {
            excelList.get(i).setCccNumCer("");
        }
        if (StringUtils.isEmpty(excelList.get(i).getPartName()) || excelList.get(i).getPartName().equals(
                "null")) {
            excelList.get(i).setPartName("");
        }
        if (StringUtils.isEmpty(excelList.get(i).getPartNum()) || excelList.get(i).getPartNum().equals(
                "null")) {
            excelList.get(i).setPartNum("");
        }
        if (StringUtils.isEmpty(excelList.get(i).getManufacturer()) || excelList.get(i).getManufacturer().equals(
                "null")) {
            excelList.get(i).setManufacturer("");
        }
    }

    private Map<String, Object> importDetails2(Sheet sheet, List<VehicleDatabaseDetailsDto> excelList) {
        if ("零部件名称".equals(String.valueOf(sheet.getRow(0).getCell(0).getStringCellValue())) &&
                "零部件型号".equals(String.valueOf(sheet.getRow(0).getCell(1).getStringCellValue())) &&
                "CCC证书编号" .equals(String.valueOf(sheet.getRow(0).getCell(2).getStringCellValue())) &&
                "生产厂家".equals(String.valueOf(sheet.getRow(0).getCell(3).getStringCellValue()))) {
            for (int i = 1; i <= sheet.getLastRowNum(); i++) {
                VehicleDatabaseDetailsDto vehicleDatabaseDetailsDto = new VehicleDatabaseDetailsDto();
                if (sheet.getRow(i) != null) {
               sheet = this.importDetails3(sheet,i);
                    if (String.valueOf(sheet.getRow(i).getCell(0)).length() > 30
                            || String.valueOf(sheet.getRow(i).getCell(1)).length() > 30
                            || String.valueOf(sheet.getRow(i).getCell(2)).length() > 30
                            || String.valueOf(sheet.getRow(i).getCell(3)).length() > 30){
                        Map<String, Object> map = new HashMap<>();
                        map.put("error", 1);
                        return map;
                    }
                    vehicleDatabaseDetailsDto.setPartName(String.valueOf(sheet.getRow(i).getCell(0)));
                    vehicleDatabaseDetailsDto.setPartNum(String.valueOf(sheet.getRow(i).getCell(1)));
                    vehicleDatabaseDetailsDto.setCccNumCer(String.valueOf(sheet.getRow(i).getCell(2)));
                    vehicleDatabaseDetailsDto.setManufacturer(String.valueOf(sheet.getRow(i).getCell(3)));
                    excelList.add(vehicleDatabaseDetailsDto);
                } else {
                    vehicleDatabaseDetailsDto.setPartName("");
                    vehicleDatabaseDetailsDto.setPartNum("");
                    vehicleDatabaseDetailsDto.setCccNumCer("");
                    vehicleDatabaseDetailsDto.setManufacturer("");
                    excelList.add(vehicleDatabaseDetailsDto);
                }
            }
        } else {
            return null;
        }
        return null;
    }

    private Sheet importDetails3(Sheet sheet, int i) {
        for (int j = 0; j <= sheet.getRow(i).getLastCellNum(); j++) {
            if (sheet.getRow(i).getCell(j) != null) {
                sheet.getRow(i).getCell(j).setCellType(Cell.CELL_TYPE_STRING);
            }
        }
        return sheet;
    }
}
